/*************************************************************************/
/*                                                                       */
/*         Header-Datei fr schnelle Bildschirm-I/O-Routinen             */
/*                                                                       */
/*                    (C) 1990-1991 Stefan Bion                          */
/*                                                                       */
/*************************************************************************/

#define true 1
#define false 0

#define AX r.x.ax
#define BX r.x.bx
#define CX r.x.cx
#define DX r.x.dx
#define AL r.h.al
#define AH r.h.ah
#define BL r.h.bl
#define BH r.h.bh
#define CL r.h.cl
#define CH r.h.ch
#define DL r.h.dl
#define DH r.h.dh
#define BP p.r_bp
#define SI p.r_si
#define DI p.r_di
#define ES s.es
#define CS s.cs
#define SS s.ss
#define DS s.ds

#define INTBIOS int86x (0x10, &r, &r, &s);
#define INTDOS int86x (0x21, &r, &r, &s);

/*************************************************************************/
/* Globale Variablen:                                                    */
/*************************************************************************/
unsigned int
    vid_segment = 0xB000,                /* Voreinstellung Herkuleskarte */
    vid_attrib = 7,                               /* Normale Darstellung */
    vid_cursor;                           /* Cursor fr put_...-Routinen */

unsigned char                    
    vid_buffer[4002],             /* Puffer fr save_- u. restore_screen */
    cursor_x,                        /* Globale Variablen fr get_cursor */
    cursor_y,                    
    in_buffer[80],                           /* Ausgabepuffer fr Zahlen */
    hexdigit[16] = "0123456789ABCDEF";          /* Array fr Hex-Ausgabe */

union   REGS r;                      /* Global fr alle Interruptaufrufe */
struct  SREGS s;
struct  REGPACK p;

/*************************************************************************/
/* Lscht den Bildschirm mit Space und dem Attribut in der globalen      */
/* Variable vid_attrib.                                                  */
/*************************************************************************/
void cls (void)
{
    AH = 6;
    AL = 0;
    CX = 0;
    DX = 0x184F;
    BH = vid_attrib;
    INTBIOS
}

/*************************************************************************/
/* Lscht Zeilen [von]...[bis] mit Space und [attrib].                   */
/*************************************************************************/
void clear_lines (int von, int bis, unsigned char attrib)
{
    int i;
    von *= 160;
    bis = bis * 160 + 160;
    for (i=von; i<bis; i=i+2)
    {
         pokeb (vid_segment, i, 0x20);
         pokeb (vid_segment, i+1, attrib);
    }
}

/*************************************************************************/
/* Speichert die Startadresse des Video-RAM's in die glob. Variable      */
/* vid_segment                                                           */
/*************************************************************************/
void video_karte (void)
{
    if (((peekb (0, 0x410) >> 4) & 3) == 3)
         vid_segment = 0xB000;
    else vid_segment = 0xB800;
}

/*************************************************************************/
/* Speichert die aktuelle Cursorposition in die globalen Variablen       */
/* cursor_x und cursor_y                                                 */
/*************************************************************************/
void get_cursor (void)
{
    AH = 3;
    BH = 0;
    INTBIOS
    cursor_x = DL;
    cursor_y = DH;
}

/*************************************************************************/
/* Setzt den Bildschirmcursor nach (x,y)                                 */
/*************************************************************************/
void set_cursor (int x, int y)
{
    AH = 2;
    BH = 0;
    DL = x;
    DH = y;
    INTBIOS
}

/*************************************************************************/
/* Definiert das Cursor-Erscheinungsbild (a = Startzeile, b = Endzeile)  */
/*************************************************************************/
void def_cursor (int a, int b)
{
    AH = 1;
    CH = a;
    CL = b;
    INTBIOS
}

/*************************************************************************/
/* Cursor-Erscheinungsbild := Block                                      */
/*************************************************************************/
void full_cursor (void)
{
    def_cursor (0, (vid_segment == 0xB000) ? 13 : 7);
}

/*************************************************************************/
/* Cursor-Erscheinungsbild := Strich                                     */
/*************************************************************************/
void norm_cursor (void)
{
    def_cursor ((vid_segment == 0xB000) ? 11 : 6, (vid_segment == 0xB000) ? 12 : 7);
}

/*************************************************************************/
/* Setzt den imaginren Cursor nach (x,y)                                */
/*************************************************************************/
void put_cursor (int x, int y)
{
    vid_cursor = 160*y + 2*x;
}

/*************************************************************************/
/* Rettet Bildschirminhalt und Cursorpos.                                */
/*************************************************************************/
void save_screen (void)
{
    unsigned int i;
    for (i=0; i<4000; i++)
         vid_buffer[i] = peekb (vid_segment, i);
    AH = 3;
    BH = 0;
    INTBIOS
    vid_buffer[4000] = DL;
    vid_buffer[4001] = DH;
}

/*************************************************************************/
/* Restauriert Bildschirm u. Cursorposition                              */
/*************************************************************************/
void restore_screen (void)
{
    unsigned int i;
    for (i=0; i<4000; i++)
         pokeb (vid_segment, i, vid_buffer[i]);
    AH = 2;
    BH = 0;
    DL = vid_buffer[4000];
    DH = vid_buffer[4001];
    INTBIOS
}

/*************************************************************************/
/* Gibt ein Zeichen an der aktuellen Position des img. Cursors aus       */
/*************************************************************************/
void put_char (unsigned char chr)
{
    pokeb (vid_segment, vid_cursor++, chr);
    pokeb (vid_segment, vid_cursor++, vid_attrib);
    if (vid_cursor == 4000)
    {
         vid_cursor -= 160;
         AX = 0x601;
         BH = vid_attrib;
         CX = 0;
         DX = 0x184F;
         INTBIOS
    }
}

/*************************************************************************/
/* Gibt einen String an der aktuellen Position des img. Cursors aus      */
/*************************************************************************/
void put_string (char *text)
{
    while (*text)
         put_char (*text++);
}

/*************************************************************************/
/* Gibt eine Hexzahl an der aktuellen Position des img. Cursors aus      */
/*************************************************************************/
void put_hex (unsigned long zahl, char stellen)
{
    char i;

    if (stellen > 8) stellen = 8;
    for (i=0; i<stellen; i++)
    {
         in_buffer[i] = hexdigit[zahl & 15];
         zahl = zahl >> 4;
    }
    for (i=stellen-1; i>=0; i--)
         put_char (in_buffer[i]);
}
